class ActionKeyVisual {
    string m_name;

	vec2 m_pos;
	vec2 m_size;

    float prev_speed = 0;
	int arr_size = 4;
	array<float> acc = {0, 0, 0, 0};
	int idx = 0;

	nvg::Font m_font;
	string m_fontPath;

	ActionKeyVisual(const string &in name)
	{
		m_name = name;
		// UpdateProportions();
        LoadFont();
	}

	void InternalRender()
	{
		vec2 screenSize = vec2(Draw::GetWidth(), Draw::GetHeight());
		vec2 pos = m_pos * (screenSize - m_size);
		nvg::Translate(pos.x, pos.y);
		Render();
		nvg::ResetTransform();
	}

	bool IsVisible(bool whenHidden) { return whenHidden ? Setting_General_ShowAccelerationHidden : Setting_General_ShowAcceleration; }
	void SetVisible(bool visible, bool visibleWhenHidden)
	{
		Setting_General_ShowAcceleration = visible;
		Setting_General_ShowAccelerationHidden = visibleWhenHidden;
	}

	void UpdateProportions()
	{
		m_pos = Setting_General_AccelerationPos;
		m_size = Setting_General_AccelerationSize;
	}

	void SetProportions(const vec2 &in pos, const vec2 &in size)
	{
		Setting_General_AccelerationPos = pos;
		Setting_General_AccelerationSize = size;
	}

	void ResetProportions()
	{
		auto plugin = Meta::ExecutingPlugin();
		plugin.GetSetting("Setting_General_AccelerationPos").Reset();
		plugin.GetSetting("Setting_General_AccelerationSize").Reset();
	}

    void LoadFont()
	{
		if (Setting_Visual_Font == m_fontPath) {
			return;
		}

		auto font = nvg::LoadFont(Setting_Visual_Font);
		if (font > 0) {
			m_fontPath = Setting_Visual_Font;
			m_font = font;
		}
	}

	void OnSettingsChanged()
	{
		LoadFont();
	}

	void Render() {
        vec2 offset = vec2(0.0f, 0.0f);
		vec2 size = m_size;

        if (currentState.currentActionKey == ActionKey::AK5 || !Setting_General_ShowAcceleration)
            return;
        RenderActionKey(offset, size, GetActionKeyString(currentState.currentActionKey));
     }

    void RenderActionKey(const vec2 &in pos, const vec2 &in size, const string &in text)
	{
		vec4 borderColor = Setting_Visual_BorderColor;
		borderColor.w *=  Setting_Visual_InactiveAlpha;

		nvg::BeginPath();
		nvg::StrokeWidth(Setting_Visual_BorderWidth);

		switch (Setting_Visual_Shape) {
			case KeyboardShape::Rectangle:
				nvg::RoundedRect(pos.x, pos.y, size.x, size.y, Setting_Visual_BorderRadius);
				break;
			case KeyboardShape::Ellipse:
				nvg::Ellipse(pos + size / 2, size.x / 2, size.y / 2);
				break;
		}

		nvg::FillColor(Setting_Visual_FillColor);
		nvg::Fill();

		nvg::StrokeColor(borderColor);
		nvg::Stroke();

		nvg::BeginPath();
		// nvg::FontFace(g_font);
		nvg::FontSize(size.x / 2);
		nvg::FillColor(borderColor);
		nvg::TextAlign(nvg::Align::Middle | nvg::Align::Center);
        nvg::FillColor(Setting_Visual_TextColor);
		nvg::TextBox(pos.x, pos.y + size.y / 2, size.x, text);
	}
}